import os
from typing import Optional

import AlteryxPythonSDK as Sdk
import pypac.parser
from pypac import PACSession

from requests.auth import HTTPProxyAuth

PROXY_ENV_HTTP_KEY = "HTTP_PROXY"
PROXY_ENV_HTTPS_KEY = "HTTPS_PROXY"
PROXY_ENV_KEYS = (PROXY_ENV_HTTP_KEY, PROXY_ENV_HTTPS_KEY)


def ayx_update_proxy_environ(*args, **kwargs) -> dict:
    previous = {k: os.environ.get(k) for k in PROXY_ENV_KEYS}
    os.environ.update(ayx_get_proxy_url_dict(*args, **kwargs))
    return previous


def ayx_restore_proxy_environ(previous: dict) -> None:
    for k, v in previous.items():
        if v is None:
            os.environ.pop(k, None)
        else:
            os.environ[k] = v


def ayx_get_proxy_url_dict(
    url: str,
    pac: Optional[pypac.parser.PACFile] = None,
    proxy_auth: Optional[HTTPProxyAuth] = None,
) -> dict:
    proxy_info = Sdk.ProxyInfo()
    if proxy_auth is None and proxy_info.auth_required:
        proxy_auth = HTTPProxyAuth(proxy_info.username, proxy_info.password)
    pac_sess = PACSession(pac=pac, proxy_auth=proxy_auth)
    if pac is None:
        pac = pac_sess.get_pac()
    if pac is None:
        proxy_url = proxy_info.get_manual_proxy(url)
        if proxy_url is None:
            proxy_url = ''
        return {k: proxy_url for k in PROXY_ENV_KEYS}
    else:
        try:
            _ = pac_sess.head(url)
        except Exception:
            # This allows failover to function inside the PAC session
            pass
        # Similar to the SalesForce Input Tool, this private attribute may change in future versions
        proxy_req_dict = pac_sess._proxy_resolver.get_proxy_for_requests(url)
        proxy_req_dict = {k: '' if v is None else v for k, v in proxy_req_dict.items()}
        return {
            PROXY_ENV_HTTP_KEY: proxy_req_dict["http"],
            PROXY_ENV_HTTPS_KEY: proxy_req_dict["https"],
        }
